==> open-xchange-documents-backend_7.8.4.orig.tar.bz2: open-xchange-documents-backend-7.8.4/com.openexchange.office.filter.odf/src/com/openexchange/office/filter/odt/dom/BodyHandler.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odt.dom;

import java.util.Stack;
import org.xml.sax.Attributes;
import com.openexchange.office.filter.odf.DLList;
import com.openexchange.office.filter.odf.ElementNSWriter;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.UnknownContentHandler;
import com.openexchange.office.filter.odf.draw.DrawFrame;
import com.openexchange.office.filter.odf.draw.DrawFrameHandler;
import com.openexchange.office.filter.odf.draw.Shape;
import com.openexchange.office.filter.odf.draw.ShapeHelper;
import com.openexchange.office.filter.odf.styles.StyleBase;
import com.openexchange.office.filter.odt.dom.components.ParagraphComponent;
import com.openexchange.office.filter.odt.dom.components.RootComponent;

public class BodyHandler extends SaxContextHandler {

	final Body body;
	final private DLList<Object> pageAnchoredComponents = new DLList<Object>();
	private Stack<SectionStartMarker> sectionStack = null;

    public BodyHandler(SaxContextHandler parentContext, Body body) {
    	super(parentContext);
    	this.body = body;
    }

    @Override
    public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName) {
    	final SaxContextHandler textHandler = TextContentHelper.startElement(this, body, attributes, uri, localName, qName);
    	if(textHandler!=null) {
    		return textHandler;
    	}
    	if(qName.equals("draw:frame")) {
    		final DrawFrame drawFrame = new DrawFrame(attributes, true);
    		pageAnchoredComponents.add(drawFrame);
    		return new DrawFrameHandler(this, drawFrame);
    	}
    	final Shape shape = ShapeHelper.getShape(attributes, uri, localName, qName, true);
    	if(shape!=null) {
    	    pageAnchoredComponents.add(shape);
    	    return shape.getContextHandler(this);
    	}
    	if(qName.equals("text:section")) {
    		if(sectionStack==null) {
    			sectionStack = new Stack<SectionStartMarker>();
    		}
    		final SectionStartMarker sectionStartMarker = new SectionStartMarker(attributes);
    		sectionStack.push(sectionStartMarker);
    		body.getContent().add(sectionStartMarker);
    		return this;
    	}
		final ElementNSWriter element = new ElementNSWriter(getFileDom(), attributes, uri, qName);
		body.getContent().add(element);
		return new UnknownContentHandler(this, element);
    }

	@Override
	public void endElement(String localName, String qName) {
		super.endElement(localName, qName);

		if(qName.equals("text:section")) {
			final SectionStartMarker sectionStartMarker = sectionStack.pop();
			final SectionEndMarker sectionEndMarker = new SectionEndMarker(sectionStartMarker);
			body.getContent().add(sectionEndMarker);
			sectionStartMarker.setEndMarker(sectionEndMarker);
		}
	}

	@Override
	public void endContext(String qName, String characters) {
		super.endContext(qName, characters);

		if(qName.equals("office:text")) {
            final Paragraph paragraph = ParagraphComponent.getFirstParagraph(new RootComponent(body), true);
            final String firstParaStyle = paragraph.getStyleName();
            if(firstParaStyle!=null&&!firstParaStyle.isEmpty()) {
                // try to determine the master page style... otherwise we will use "Standard"
                final StyleBase paraStyle = ((TextContent)body.getOwnerDocument()).getDocument().getStyleManager().getStyle(firstParaStyle, "paragraph", true);
                if(paraStyle!=null) {
                    final String masterPageName = paraStyle.getAttribute("style:master-page-name");
                    if(masterPageName!=null) {
                        body.setMasterPageName(masterPageName);
                    }
                }
            }
			if(!pageAnchoredComponents.isEmpty()) {
				// move these objects into the first paragraph...
				final TextSpan textSpan = new TextSpan();
				paragraph.getContent().addFirst(textSpan);
				textSpan.setContent(pageAnchoredComponents);
			}
		}
	}
}

==> open-xchange-documents-backend_7.10.1.orig.tar.bz2: open-xchange-documents-backend-7.10.1/com.openexchange.office.filter.odf/src/com/openexchange/office/filter/odt/dom/BodyHandler.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odt.dom;

import java.util.Stack;
import org.xml.sax.Attributes;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.DLList;
import com.openexchange.office.filter.odf.ElementNSWriter;
import com.openexchange.office.filter.odf.IContentDom;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.UnknownContentHandler;
import com.openexchange.office.filter.odf.draw.DrawFrame;
import com.openexchange.office.filter.odf.draw.DrawFrameHandler;
import com.openexchange.office.filter.odf.draw.Shape;
import com.openexchange.office.filter.odf.draw.ShapeHelper;
import com.openexchange.office.filter.odf.styles.StyleBase;
import com.openexchange.office.filter.odf.styles.StyleFamily;
import com.openexchange.office.filter.odt.dom.components.ParagraphComponent;
import com.openexchange.office.filter.odt.dom.components.RootComponent;

public class BodyHandler extends SaxContextHandler {

	final Body body;
	final boolean contentStyle;
	final private DLList<Object> pageAnchoredComponents = new DLList<Object>();
	private Stack<SectionStartMarker> sectionStack = null;

    public BodyHandler(SaxContextHandler parentContext, Body body) {
    	super(parentContext);
    	this.body = body;
    	contentStyle = getFileDom() instanceof IContentDom;
    }

    @Override
    public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName) {
    	final SaxContextHandler textHandler = TextContentHelper.startElement(this, body, attributes, uri, localName, qName);
    	if(textHandler!=null) {
    		return textHandler;
    	}
    	if(qName.equals("draw:frame")) {
    		final DrawFrame drawFrame = new DrawFrame(new AttributesImpl(attributes), null, true, contentStyle);
    		pageAnchoredComponents.add(drawFrame);
    		return new DrawFrameHandler(this, drawFrame);
    	}
    	final Shape shape = ShapeHelper.getShape(attributes, uri, localName, qName, null, true, contentStyle);
    	if(shape!=null) {
    	    pageAnchoredComponents.add(shape);
    	    return shape.getContextHandler(this);
    	}
    	if(qName.equals("text:section")) {
    		if(sectionStack==null) {
    			sectionStack = new Stack<SectionStartMarker>();
    		}
    		final SectionStartMarker sectionStartMarker = new SectionStartMarker(attributes);
    		sectionStack.push(sectionStartMarker);
    		body.getContent().add(sectionStartMarker);
    		return this;
    	}
		final ElementNSWriter element = new ElementNSWriter(getFileDom(), attributes, uri, qName);
		body.getContent().add(element);
		return new UnknownContentHandler(this, element);
    }

	@Override
	public void endElement(String localName, String qName) {
		super.endElement(localName, qName);

		if(qName.equals("text:section")) {
			final SectionStartMarker sectionStartMarker = sectionStack.pop();
			final SectionEndMarker sectionEndMarker = new SectionEndMarker(sectionStartMarker);
			body.getContent().add(sectionEndMarker);
			sectionStartMarker.setEndMarker(sectionEndMarker);
		}
	}

	@Override
	public void endContext(String qName, String characters) {
		super.endContext(qName, characters);

		if(qName.equals("office:text")) {
            final Paragraph paragraph = ParagraphComponent.getFirstParagraph(new RootComponent(body, true), true);
            final String firstParaStyle = paragraph.getStyleName();
            if(firstParaStyle!=null&&!firstParaStyle.isEmpty()) {
                // try to determine the master page style... otherwise we will use "Standard"
                final StyleBase paraStyle = ((TextContent)body.getOwnerDocument()).getDocument().getStyleManager().getStyle(firstParaStyle, StyleFamily.PARAGRAPH, true);
                if(paraStyle!=null) {
                    final String masterPageName = paraStyle.getAttribute("style:master-page-name");
                    if(masterPageName!=null) {
                        body.setMasterPageName(masterPageName);
                    }
                }
            }
			if(!pageAnchoredComponents.isEmpty()) {
				// move these objects into the first paragraph...
				final TextSpan textSpan = new TextSpan();
				paragraph.getContent().addFirst(textSpan);
				textSpan.setContent(pageAnchoredComponents);
			}
		}
	}
}
